@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2010, Roger Light <roger@atchoo.org>
+Copyright (c) 2010-2012 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,16 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <time.h>
 #include <mosquitto.h>
 
-namespace mosquittopp {
+namespace mosqpp {
+
+const char *strerror(int mosq_errno);
+const char *connack_string(int connack_code);
+int sub_topic_tokenise(const char *subtopic, char ***topics, int *count);
+int sub_topic_tokens_free(char ***topics, int count);
+int lib_version(int *major, int *minor, int *revision);
+int lib_init();
+int lib_cleanup();
+int topic_matches_sub(const char *sub, const char *topic, bool *result);
 
 /*
  * Class: mosquittopp
@@ -54,37 +63,44 @@ namespace mosquittopp {
  */
 class mosqpp_EXPORT mosquittopp {
 	private:
-		struct mosquitto *mosq;
+		struct mosquitto *m_mosq;
 	public:
-		mosquittopp(const char *id);
+		mosquittopp(const char *id=NULL, bool clean_session=true);
 		~mosquittopp();
 
-		static void lib_version(int *major, int *minor, int *revision);
-		static int lib_init();
-		static int lib_cleanup();
+		int reinitialise(const char *id, bool clean_session);
 		int socket();
-		int log_init(int priorities, int destinations);
-		int will_set(bool will, const char *topic, uint32_t payloadlen=0, const uint8_t *payload=NULL, int qos=0, bool retain=false);
+		int will_set(const char *topic, int payloadlen=0, const void *payload=NULL, int qos=0, bool retain=false);
+		int will_clear();
 		int username_pw_set(const char *username, const char *password=NULL);
-		int connect(const char *host, int port=1883, int keepalive=60, bool clean_session=true);
+		int connect(const char *host, int port=1883, int keepalive=60);
+		int connect_async(const char *host, int port=1883, int keepalive=60);
 		int reconnect();
 		int disconnect();
-		int publish(uint16_t *mid, const char *topic, uint32_t payloadlen=0, const uint8_t *payload=NULL, int qos=0, bool retain=false);
-		int subscribe(uint16_t *mid, const char *sub, int qos=0);
-		int unsubscribe(uint16_t *mid, const char *sub);
+		int publish(int *mid, const char *topic, int payloadlen=0, const void *payload=NULL, int qos=0, bool retain=false);
+		int subscribe(int *mid, const char *sub, int qos=0);
+		int unsubscribe(int *mid, const char *sub);
 		void message_retry_set(unsigned int message_retry);
+		void user_data_set(void *obj);
+		int tls_set(const char *cafile, const char *capath=NULL, const char *certfile=NULL, const char *keyfile=NULL, int (*pw_callback)(char *buf, int size, int rwflag, void *userdata)=NULL);
+		int tls_opts_set(int cert_reqs, const char *tls_version=NULL, const char *ciphers=NULL);
+		int tls_psk_set(const char *psk, const char *identity, const char *ciphers=NULL);
 
-		int loop(int timeout=-1);
+		int loop(int timeout=-1, int max_packets=1);
 		int loop_misc();
-		int loop_read();
-		int loop_write();
+		int loop_read(int max_packets=1);
+		int loop_write(int max_packets=1);
+		int loop_start();
+		int loop_stop(bool force=false);
+		bool want_write();
 		
 		virtual void on_connect(int rc) {return;};
-		virtual void on_disconnect() {return;};
-		virtual void on_publish(uint16_t mid) {return;};
+		virtual void on_disconnect(int rc) {return;};
+		virtual void on_publish(int mid) {return;};
 		virtual void on_message(const struct mosquitto_message *message) {return;};
-		virtual void on_subscribe(uint16_t mid, int qos_count, const uint8_t *granted_qos) {return;};
-		virtual void on_unsubscribe(uint16_t mid) {return;};
+		virtual void on_subscribe(int mid, int qos_count, const int *granted_qos) {return;};
+		virtual void on_unsubscribe(int mid) {return;};
+		virtual void on_log(int level, const char *str) {return;};
 		virtual void on_error() {return;};
 };
 
