@@ -47,16 +47,18 @@ int _mosquitto_handle_connack(struct mosquitto *mosq)
 		return MOSQ_ERR_PROTOCOL;
 	}
 #endif
-	_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Received CONNACK");
+	_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Client %s received CONNACK", mosq->id);
 	rc = _mosquitto_read_byte(&mosq->in_packet, &byte); // Reserved byte, not used
 	if(rc) return rc;
 	rc = _mosquitto_read_byte(&mosq->in_packet, &result);
 	if(rc) return rc;
+	pthread_mutex_lock(&mosq->callback_mutex);
 	if(mosq->on_connect){
 		mosq->in_callback = true;
-		mosq->on_connect(mosq->obj, result);
+		mosq->on_connect(mosq, mosq->obj, result);
 		mosq->in_callback = false;
 	}
+	pthread_mutex_unlock(&mosq->callback_mutex);
 	switch(result){
 		case 0:
 			mosq->state = mosq_cs_connected;
