@@ -33,7 +33,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 int _mosquitto_send_simple_command(struct mosquitto *mosq, uint8_t command);
 int _mosquitto_send_command_with_mid(struct mosquitto *mosq, uint8_t command, uint16_t mid, bool dup);
-int _mosquitto_send_real_publish(struct mosquitto *mosq, uint16_t mid, const char *topic, uint32_t payloadlen, const uint8_t *payload, int qos, bool retain, bool dup);
+int _mosquitto_send_real_publish(struct mosquitto *mosq, uint16_t mid, const char *topic, uint32_t payloadlen, const void *payload, int qos, bool retain, bool dup);
 
 int _mosquitto_send_connect(struct mosquitto *mosq, uint16_t keepalive, bool clean_session);
 int _mosquitto_send_disconnect(struct mosquitto *mosq);
@@ -41,10 +41,10 @@ int _mosquitto_send_pingreq(struct mosquitto *mosq);
 int _mosquitto_send_pingresp(struct mosquitto *mosq);
 int _mosquitto_send_puback(struct mosquitto *mosq, uint16_t mid);
 int _mosquitto_send_pubcomp(struct mosquitto *mosq, uint16_t mid);
-int _mosquitto_send_publish(struct mosquitto *mosq, uint16_t mid, const char *topic, uint32_t payloadlen, const uint8_t *payload, int qos, bool retain, bool dup);
+int _mosquitto_send_publish(struct mosquitto *mosq, uint16_t mid, const char *topic, uint32_t payloadlen, const void *payload, int qos, bool retain, bool dup);
 int _mosquitto_send_pubrec(struct mosquitto *mosq, uint16_t mid);
 int _mosquitto_send_pubrel(struct mosquitto *mosq, uint16_t mid, bool dup);
-int _mosquitto_send_subscribe(struct mosquitto *mosq, uint16_t *mid, bool dup, const char *topic, uint8_t topic_qos);
-int _mosquitto_send_unsubscribe(struct mosquitto *mosq, uint16_t *mid, bool dup, const char *topic);
+int _mosquitto_send_subscribe(struct mosquitto *mosq, int *mid, bool dup, const char *topic, uint8_t topic_qos);
+int _mosquitto_send_unsubscribe(struct mosquitto *mosq, int *mid, bool dup, const char *topic);
 
 #endif
