@@ -31,8 +31,9 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <stdio.h>
 #include <string.h>
 
-#include <mqtt3.h>
+#include <mosquitto_broker.h>
 #include <memory_mosq.h>
+#include <mqtt3_protocol.h>
 #include <send_mosq.h>
 #include <util_mosq.h>
 
@@ -43,7 +44,7 @@ int mqtt3_handle_connack(mosquitto_db *db, struct mosquitto *context)
 	int i;
 	char *notification_topic;
 	int notification_topic_len;
-	uint8_t notification_payload[2];
+	char notification_payload[2];
 
 	if(!context){
 		return MOSQ_ERR_INVAL;
@@ -57,28 +58,46 @@ int mqtt3_handle_connack(mosquitto_db *db, struct mosquitto *context)
 	if(_mosquitto_read_byte(&context->in_packet, &byte)) return 1; // Reserved byte, not used
 	if(_mosquitto_read_byte(&context->in_packet, &rc)) return 1;
 	switch(rc){
-		case 0:
+		case CONNACK_ACCEPTED:
 			if(context->bridge){
 				if(context->bridge->notifications){
-					notification_topic_len = strlen(context->id)+strlen("$SYS/broker/connection//state");
-					notification_topic = _mosquitto_malloc(sizeof(char)*(notification_topic_len+1));
-					if(!notification_topic) return MOSQ_ERR_NOMEM;
-
-					snprintf(notification_topic, notification_topic_len+1, "$SYS/broker/connection/%s/state", context->id);
 					notification_payload[0] = '1';
 					notification_payload[1] = '\0';
-					if(_mosquitto_send_real_publish(context, _mosquitto_mid_generate(context),
-							notification_topic, 2, (uint8_t *)&notification_payload, 1, true, 0)){
+					if(context->bridge->notification_topic){
+						if(_mosquitto_send_real_publish(context, _mosquitto_mid_generate(context),
+								context->bridge->notification_topic, 2, &notification_payload, 1, true, 0)){
 
+							return 1;
+						}
+						mqtt3_db_messages_easy_queue(db, context, context->bridge->notification_topic, 1, 2, &notification_payload, 1);
+					}else{
+						notification_topic_len = strlen(context->id)+strlen("$SYS/broker/connection//state");
+						notification_topic = _mosquitto_malloc(sizeof(char)*(notification_topic_len+1));
+						if(!notification_topic) return MOSQ_ERR_NOMEM;
+
+						snprintf(notification_topic, notification_topic_len+1, "$SYS/broker/connection/%s/state", context->id);
+						notification_payload[0] = '1';
+						notification_payload[1] = '\0';
+						if(_mosquitto_send_real_publish(context, _mosquitto_mid_generate(context),
+								notification_topic, 2, &notification_payload, 1, true, 0)){
+
+							_mosquitto_free(notification_topic);
+							return 1;
+						}
+						mqtt3_db_messages_easy_queue(db, context, notification_topic, 1, 2, &notification_payload, 1);
 						_mosquitto_free(notification_topic);
-						return 1;
 					}
-					mqtt3_db_messages_easy_queue(db, context, notification_topic, 1, 2, (uint8_t *)&notification_payload, 1);
-					_mosquitto_free(notification_topic);
 				}
 				for(i=0; i<context->bridge->topic_count; i++){
 					if(context->bridge->topics[i].direction == bd_in || context->bridge->topics[i].direction == bd_both){
-						if(_mosquitto_send_subscribe(context, NULL, false, context->bridge->topics[i].topic, 2)){
+						if(_mosquitto_send_subscribe(context, NULL, false, context->bridge->topics[i].remote_topic, context->bridge->topics[i].qos)){
+							return 1;
+						}
+					}else{
+						if(_mosquitto_send_unsubscribe(context, NULL, false, context->bridge->topics[i].remote_topic)){
+							/* direction = inwards only. This means we should not be subscribed
+			 				 * to the topic. It is possible that we used to be subscribed to
+			 				 * this topic so unsubscribe. */
 							return 1;
 						}
 					}
@@ -86,15 +105,27 @@ int mqtt3_handle_connack(mosquitto_db *db, struct mosquitto *context)
 			}
 			context->state = mosq_cs_connected;
 			return MOSQ_ERR_SUCCESS;
-		case 1:
+		case CONNACK_REFUSED_PROTOCOL_VERSION:
+			if(context->bridge){
+				context->bridge->try_private_accepted = false;
+			}
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Connection Refused: unacceptable protocol version");
 			return 1;
-		case 2:
+		case CONNACK_REFUSED_IDENTIFIER_REJECTED:
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Connection Refused: identifier rejected");
 			return 1;
-		case 3:
+		case CONNACK_REFUSED_SERVER_UNAVAILABLE:
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Connection Refused: broker unavailable");
 			return 1;
+		case CONNACK_REFUSED_BAD_USERNAME_PASSWORD:
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Connection Refused: broker unavailable");
+			return 1;
+		case CONNACK_REFUSED_NOT_AUTHORIZED:
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Connection Refused: not authorised");
+			return 1;
+		default:
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Connection Refused: unknown reason");
+			return 1;
 	}
 	return 1;
 }
