@@ -32,19 +32,20 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include <config.h>
 
-#include <mqtt3.h>
+#include <mosquitto_broker.h>
 #include <mqtt3_protocol.h>
 #include <memory_mosq.h>
 #include <send_mosq.h>
 #include <util_mosq.h>
 
-int mqtt3_handle_connect(mosquitto_db *db, int context_index)
+int mqtt3_handle_connect(mosquitto_db *db, struct mosquitto *context)
 {
 	char *protocol_name;
 	uint8_t protocol_version;
 	uint8_t connect_flags;
 	char *client_id;
-	char *will_message = NULL, *will_topic = NULL;
+	char *will_payload = NULL, *will_topic = NULL;
+	uint16_t will_payloadlen;
 	struct mosquitto_message *will_struct = NULL;
 	uint8_t will, will_retain, will_qos, clean_session;
 	uint8_t username_flag, password_flag;
@@ -52,22 +53,25 @@ int mqtt3_handle_connect(mosquitto_db *db, int context_index)
 	int i;
 	int rc;
 	struct _mosquitto_acl_user *acl_tail;
-	struct mosquitto *context;
+	int slen;
+#ifdef WITH_TLS
+	X509 *client_cert;
+	X509_NAME *name;
+	X509_NAME_ENTRY *name_entry;
+#endif
 
-	context = db->contexts[context_index];
-	
 	/* Don't accept multiple CONNECT commands. */
 	if(context->state != mosq_cs_new){
-		mqtt3_context_disconnect(db, context_index);
+		mqtt3_context_disconnect(db, context);
 		return MOSQ_ERR_PROTOCOL;
 	}
 
 	if(_mosquitto_read_string(&context->in_packet, &protocol_name)){
-		mqtt3_context_disconnect(db, context_index);
+		mqtt3_context_disconnect(db, context);
 		return 1;
 	}
 	if(!protocol_name){
-		mqtt3_context_disconnect(db, context_index);
+		mqtt3_context_disconnect(db, context);
 		return 3;
 	}
 	if(strcmp(protocol_name, PROTOCOL_NAME)){
@@ -76,28 +80,30 @@ int mqtt3_handle_connect(mosquitto_db *db, int context_index)
 					protocol_name, context->address);
 		}
 		_mosquitto_free(protocol_name);
-		mqtt3_context_disconnect(db, context_index);
+		mqtt3_context_disconnect(db, context);
 		return MOSQ_ERR_PROTOCOL;
 	}
 	_mosquitto_free(protocol_name);
 
 	if(_mosquitto_read_byte(&context->in_packet, &protocol_version)){
-		mqtt3_context_disconnect(db, context_index);
+		mqtt3_context_disconnect(db, context);
 		return 1;
 	}
-	if(protocol_version != PROTOCOL_VERSION){
+	if((protocol_version&0x7F) != PROTOCOL_VERSION){
 		if(db->config->connection_messages == true){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_INFO, "Invalid protocol version %d in CONNECT from %s.",
 					protocol_version, context->address);
 		}
-		_mosquitto_free(protocol_name);
-		_mosquitto_send_connack(context, 1);
-		mqtt3_context_disconnect(db, context_index);
+		_mosquitto_send_connack(context, CONNACK_REFUSED_PROTOCOL_VERSION);
+		mqtt3_context_disconnect(db, context);
 		return MOSQ_ERR_PROTOCOL;
 	}
+	if((protocol_version&0x80) == 0x80){
+		context->is_bridge = true;
+	}
 
 	if(_mosquitto_read_byte(&context->in_packet, &connect_flags)){
-		mqtt3_context_disconnect(db, context_index);
+		mqtt3_context_disconnect(db, context);
 		return 1;
 	}
 	clean_session = connect_flags & 0x02;
@@ -108,12 +114,23 @@ int mqtt3_handle_connect(mosquitto_db *db, int context_index)
 	username_flag = connect_flags & 0x80;
 
 	if(_mosquitto_read_uint16(&context->in_packet, &(context->keepalive))){
-		mqtt3_context_disconnect(db, context_index);
+		mqtt3_context_disconnect(db, context);
 		return 1;
 	}
 
 	if(_mosquitto_read_string(&context->in_packet, &client_id)){
-		mqtt3_context_disconnect(db, context_index);
+		mqtt3_context_disconnect(db, context);
+		return 1;
+	}
+
+	slen = strlen(client_id);
+#ifdef WITH_STRICT_PROTOCOL
+	if(slen > 23 || slen == 0){
+#else
+	if(slen == 0){
+#endif
+		_mosquitto_send_connack(context, CONNACK_REFUSED_IDENTIFIER_REJECTED);
+		mqtt3_context_disconnect(db, context);
 		return 1;
 	}
 
@@ -121,8 +138,8 @@ int mqtt3_handle_connect(mosquitto_db *db, int context_index)
 	if(db->config->clientid_prefixes){
 		if(strncmp(db->config->clientid_prefixes, client_id, strlen(db->config->clientid_prefixes))){
 			_mosquitto_free(client_id);
-			_mosquitto_send_connack(context, 2);
-			mqtt3_context_disconnect(db, context_index);
+			_mosquitto_send_connack(context, CONNACK_REFUSED_NOT_AUTHORIZED);
+			mqtt3_context_disconnect(db, context);
 			return MOSQ_ERR_SUCCESS;
 		}
 	}
@@ -131,17 +148,31 @@ int mqtt3_handle_connect(mosquitto_db *db, int context_index)
 		will_struct = _mosquitto_calloc(1, sizeof(struct mosquitto_message));
 		if(!will_struct){
 			_mosquitto_free(client_id);
-			mqtt3_context_disconnect(db, context_index);
+			mqtt3_context_disconnect(db, context);
 			return MOSQ_ERR_NOMEM;
 		}
 		if(_mosquitto_read_string(&context->in_packet, &will_topic)){
 			_mosquitto_free(client_id);
-			mqtt3_context_disconnect(db, context_index);
+			mqtt3_context_disconnect(db, context);
 			return 1;
 		}
-		if(_mosquitto_read_string(&context->in_packet, &will_message)){
+		if(_mosquitto_read_uint16(&context->in_packet, &will_payloadlen)){
 			_mosquitto_free(client_id);
-			mqtt3_context_disconnect(db, context_index);
+			mqtt3_context_disconnect(db, context);
+			return 1;
+		}
+		will_payload = _mosquitto_malloc(will_payloadlen);
+		if(!will_payload){
+			_mosquitto_free(client_id);
+			mqtt3_context_disconnect(db, context);
+			return 1;
+		}
+
+		rc = _mosquitto_read_bytes(&context->in_packet, will_payload, will_payloadlen);
+		if(rc){
+			_mosquitto_free(will_payload);
+			_mosquitto_free(client_id);
+			mqtt3_context_disconnect(db, context);
 			return 1;
 		}
 	}
@@ -160,18 +191,6 @@ int mqtt3_handle_connect(mosquitto_db *db, int context_index)
 					password_flag = 0;
 				}
 			}
-			rc = mosquitto_unpwd_check(db, username, password);
-			context->username = username;
-			context->password = password;
-			if(rc == MOSQ_ERR_AUTH){
-				_mosquitto_send_connack(context, 2);
-				mqtt3_context_disconnect(db, context_index);
-				_mosquitto_free(client_id);
-				return MOSQ_ERR_SUCCESS;
-			}else if(rc == MOSQ_ERR_INVAL){
-				_mosquitto_free(client_id);
-				return MOSQ_ERR_INVAL;
-			}
 		}else if(rc == MOSQ_ERR_NOMEM){
 			_mosquitto_free(client_id);
 			return MOSQ_ERR_NOMEM;
@@ -181,12 +200,70 @@ int mqtt3_handle_connect(mosquitto_db *db, int context_index)
 		}
 	}
 
-	if(!username_flag && db->config->allow_anonymous == false){
-		_mosquitto_send_connack(context, 2);
-		mqtt3_context_disconnect(db, context_index);
-		_mosquitto_free(client_id);
-		return MOSQ_ERR_SUCCESS;
+#ifdef WITH_TLS
+	if(context->listener->use_identity_as_username){
+		if(!context->ssl){
+			_mosquitto_send_connack(context, CONNACK_REFUSED_BAD_USERNAME_PASSWORD);
+			mqtt3_context_disconnect(db, context);
+			_mosquitto_free(client_id);
+			return MOSQ_ERR_SUCCESS;
+		}
+#ifdef WITH_TLS_PSK
+		if(context->listener->psk_hint){
+			/* Client should have provided an identity to get this far. */
+			if(!context->username){
+				_mosquitto_send_connack(context, CONNACK_REFUSED_BAD_USERNAME_PASSWORD);
+				mqtt3_context_disconnect(db, context);
+				_mosquitto_free(client_id);
+				return MOSQ_ERR_SUCCESS;
+			}
+		}else{
+#endif /* WITH_TLS_PSK */
+			client_cert = SSL_get_peer_certificate(context->ssl);
+			name = X509_get_subject_name(client_cert);
+
+			i = X509_NAME_get_index_by_NID(name, NID_commonName, -1);
+			if(i == -1){
+				_mosquitto_send_connack(context, CONNACK_REFUSED_BAD_USERNAME_PASSWORD);
+				mqtt3_context_disconnect(db, context);
+				_mosquitto_free(client_id);
+				return MOSQ_ERR_SUCCESS;
+			}
+			name_entry = X509_NAME_get_entry(name, i);
+			context->username = _mosquitto_strdup((char *)ASN1_STRING_data(name_entry->value));
+			if(!context->username){
+				_mosquitto_free(client_id);
+				return MOSQ_ERR_NOMEM;
+			}
+#ifdef WITH_TLS_PSK
+		}
+#endif /* WITH_TLS_PSK */
+	}else{
+#endif /* WITH_TLS */
+		if(username_flag){
+			rc = mosquitto_unpwd_check(db, username, password);
+			context->username = username;
+			context->password = password;
+			if(rc == MOSQ_ERR_AUTH){
+				_mosquitto_send_connack(context, CONNACK_REFUSED_BAD_USERNAME_PASSWORD);
+				mqtt3_context_disconnect(db, context);
+				_mosquitto_free(client_id);
+				return MOSQ_ERR_SUCCESS;
+			}else if(rc == MOSQ_ERR_INVAL){
+				_mosquitto_free(client_id);
+				return MOSQ_ERR_INVAL;
+			}
+		}
+
+		if(!username_flag && db->config->allow_anonymous == false){
+			_mosquitto_send_connack(context, CONNACK_REFUSED_NOT_AUTHORIZED);
+			mqtt3_context_disconnect(db, context);
+			_mosquitto_free(client_id);
+			return MOSQ_ERR_SUCCESS;
+		}
+#ifdef WITH_TLS
 	}
+#endif
 
 	/* Find if this client already has an entry. This must be done *after* any security checks. */
 	for(i=0; i<db->context_count; i++){
@@ -210,32 +287,21 @@ int mqtt3_handle_connect(mosquitto_db *db, int context_index)
 			db->contexts[i]->last_msg_in = time(NULL);
 			db->contexts[i]->last_msg_out = time(NULL);
 			db->contexts[i]->keepalive = context->keepalive;
+			db->contexts[i]->pollfd_index = context->pollfd_index;
+#ifdef WITH_TLS
+			db->contexts[i]->ssl = context->ssl;
+#endif
+			if(context->username){
+				db->contexts[i]->username = _mosquitto_strdup(context->username);
+			}
 			context->sock = -1;
+#ifdef WITH_TLS
+			context->ssl = NULL;
+#endif
 			context->state = mosq_cs_disconnecting;
 			context = db->contexts[i];
 			if(context->msgs){
-				/* Messages received when the client was disconnected are put
-				 * in the ms_queued state. If we don't change them to the
-				 * appropriate "publish" state, then the queued messages won't
-				 * get sent until the client next receives a message - and they
-				 * will be sent out of order.
-				 * This only sets a single message up to be published, but once
-				 * it is sent the full max_inflight amount of messages will be
-				 * queued up for sending.
-				 */
-				if(context->msgs->state == ms_queued && context->msgs->direction == mosq_md_out){
-					switch(context->msgs->qos){
-						case 0:
-							context->msgs->state = ms_publish;
-							break;
-						case 1:
-							context->msgs->state = ms_publish_puback;
-							break;
-						case 2:
-							context->msgs->state = ms_publish_pubrec;
-							break;
-					}
-				}
+				mqtt3_db_message_reconnect_reset(context);
 			}
 			break;
 		}
@@ -243,13 +309,19 @@ int mqtt3_handle_connect(mosquitto_db *db, int context_index)
 
 	context->id = client_id;
 	context->clean_session = clean_session;
+	context->ping_t = 0;
 
+#ifdef WITH_PERSISTENCE
+	if(!clean_session){
+		db->persistence_changes++;
+	}
+#endif
 	context->will = will_struct;
 	if(context->will){
 		context->will->topic = will_topic;
-		if(will_message){
-			context->will->payload = (uint8_t *)will_message;
-			context->will->payloadlen = strlen(will_message);
+		if(will_payload){
+			context->will->payload = will_payload;
+			context->will->payloadlen = will_payloadlen;
 		}else{
 			context->will->payload = NULL;
 			context->will->payloadlen = 0;
@@ -284,15 +356,11 @@ int mqtt3_handle_connect(mosquitto_db *db, int context_index)
 	}
 
 	context->state = mosq_cs_connected;
-	return _mosquitto_send_connack(context, 0);
+	return _mosquitto_send_connack(context, CONNACK_ACCEPTED);
 }
 
-int mqtt3_handle_disconnect(mosquitto_db *db, int context_index)
+int mqtt3_handle_disconnect(mosquitto_db *db, struct mosquitto *context)
 {
-	struct mosquitto *context;
-
-	context = db->contexts[context_index];
-
 	if(!context){
 		return MOSQ_ERR_INVAL;
 	}
@@ -301,7 +369,7 @@ int mqtt3_handle_disconnect(mosquitto_db *db, int context_index)
 	}
 	_mosquitto_log_printf(NULL, MOSQ_LOG_DEBUG, "Received DISCONNECT from %s", context->id);
 	context->state = mosq_cs_disconnecting;
-	mqtt3_context_disconnect(db, context_index);
+	mqtt3_context_disconnect(db, context);
 	return MOSQ_ERR_SUCCESS;
 }
 
@@ -375,13 +443,13 @@ int mqtt3_handle_subscribe(mosquitto_db *db, struct mosquitto *context)
 			/* Check for topic access */
 			rc2 = mqtt3_acl_check(db, context, sub, MOSQ_ACL_READ);
 			if(rc2 == MOSQ_ERR_SUCCESS){
-				mqtt3_sub_add(context, sub, qos, &db->subs);
+				mqtt3_sub_add(db, context, sub, qos, &db->subs);
 				if(mqtt3_retain_queue(db, context, sub, qos)) rc = 1;
 			}else if(rc2 != MOSQ_ERR_ACL_DENIED){
 				rc = 1;
 			}
 #else
-			rc2 = mqtt3_sub_add(context, sub, qos, &db->subs);
+			rc2 = mqtt3_sub_add(db, context, sub, qos, &db->subs);
 			if(rc2 == MOSQ_ERR_SUCCESS){
 				if(mqtt3_retain_queue(db, context, sub, qos)) rc = 1;
 			}else if(rc2 != -1){
@@ -406,6 +474,10 @@ int mqtt3_handle_subscribe(mosquitto_db *db, struct mosquitto *context)
 	if(_mosquitto_send_suback(context, mid, payloadlen, payload)) rc = 1;
 	_mosquitto_free(payload);
 	
+#ifdef WITH_PERSISTENCE
+	db->persistence_changes++;
+#endif
+
 	return rc;
 }
 
@@ -427,10 +499,13 @@ int mqtt3_handle_unsubscribe(mosquitto_db *db, struct mosquitto *context)
 
 		if(sub){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_DEBUG, "\t%s", sub);
-			mqtt3_sub_remove(context, sub, &db->subs);
+			mqtt3_sub_remove(db, context, sub, &db->subs);
 			_mosquitto_free(sub);
 		}
 	}
+#ifdef WITH_PERSISTENCE
+	db->persistence_changes++;
+#endif
 
 	return _mosquitto_send_command_with_mid(context, UNSUBACK, mid, false);
 }
